<?php

/************************************************************************/
/* Copyright Russell McGuire 2001-2007                                  */
/* Provided under Living Stones Software License Agreement              */
/* By using or sharing this software with others, you agree to be bound */
/* by the terms of that agreement.                                      */
/* E-mail russ.mcguire@seek-first.com if you don't have a copy          */
/************************************************************************/

/************************************************************************/
/* Living Stones Common Functions                                       */
/*                                                                      */
/*----------------------------------------------------------------------*/
/* Authorization functions - a generic set of functions for controlling */
/*                         access to Living Stones building block data  */
/*                                                                      */
/* If you want to use a different security mechanism, you can probably  */
/* modify these functions without having to change all the other        */
/* Living Stones scripts.                                               */
/*----------------------------------------------------------------------*/
/*                                                                      */
/* set_auth() - Define the level of authorization a user has to         */
/*                     work with a specific object                      */
/*                                                                      */
/* get_auth() - Check the level of authorization a user has to          */
/*                     work with a specific object                      */
/*                                                                      */
/* check_auth() - Check that a user has at least a specified            */
/*                       level of authorization to work with            */
/*                       a specific object                              */
/*                                                                      */
/* check_function_auth() - Check that a user has at least a specified   */
/*                       level of authorization to perform a            */
/*                       specific function                              */
/*                                                                      */
/* check_type_auth() - Check that a user has at least a specified       */
/*                       level of authorization to work with            */
/*                       a specific type of object (incl. functions)    */
/*                                                                      */
/* create_admin_menu() - Based on access to different types of objects, */
/*                       create the admin menu to manage those objects. */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Explanation of security structure                                    */
/*                                                                      */
/* In order to handle security for an as yet undetermined set of LS     */
/* building blocks that might be installed at a specific website,       */
/* the Living Stones security structure has been defined in a very      */
/* generic and flexible manner.                                         */
/*                                                                      */
/* For starters, security is based on the general concept of 'objects'  */
/* which are of different types.  For example, we likely want to        */
/* control access to user data.  Therefore, there will be an object     */
/* type of "users" and there will be specific objects which are the     */
/* individual users.  Each object of a certain type will have an "id",  */
/* which typically will be the index into the database table holding    */
/* the core data about that type of object (e.g. the index into the     */
/* table of users).   [The table ls_security_objects makes the link     */
/* between building blocks and their database tables and the security   */
/* framework.  The table ls_security_auth ties individual users to the  */
/* level of access they have to each individual object of each type.]   */
/*                                                                      */
/* The second concept is of authorization levels.  Instead of           */
/* hardcoding authorization levels into all the Living Stones software, */
/* we've set it up where anyone who adds building blocks to their       */
/* website can choose to handle things differently.  One website might  */
/* want "members" to be able to view everything about any user, while   */
/* another site might want "members" to only be able to view names and  */
/* telephone numbers.  In any case, Living Stones blocks will come with */
/* a set of prespecified security levels for each new object type.      */
/* These can be modified and new levels can be added.  However, you     */
/* probably want to make sure that higher numerical values mean higher  */
/* levels of security/access. [The table ls_security_levels defines the */
/* different levels of security that can be applied to a type of        */
/* object.]                                                             */
/*                                                                      */
/* An important aspect of this is defining the access on a field by     */
/* field basis for an object with a security level.  For each level for */
/* each object type access to individual database fields for the object */
/* can be defined as no access, read only, or read+write.  [The table   */
/* ls_security_fields defines the access for each field for each        */
/* security level.]                                                     */
/*                                                                      */
/* The third concept is of functions.  In addition to controlling       */
/* access to specific objects within Living Stones building blocks, we  */
/* also want to control who can do certain types of things, such as     */
/* upgrade to a newer version of a building block, or add a new user.   */
/* These are handled by treating these general functions as objects     */
/* themselves (with object type of zero). [The table                    */
/* ls_security_functions defines these functions.]                      */
/*                                                                      */
/* Finally, it's important to understand how the security for any given */
/* user is determined.  Living Stones will support the idea of groups   */
/* of users.  A user's level of access to any object or function will   */
/* be the highest of any level of access either set specifically for    */
/* this user, set specifically for a group this user is a member of, or */
/* set for the group of "all users."                                    */
/*                                                                      */
/* Within all of this, it's important to know what certain values mean. */
/*                                                                      */
/* Since a higher value for authorization level means greater access,   */
/* an authorization level = 0 means No Access.                          */
/*                                                                      */
/* An entry in the security databases where userid = 0 defines the      */
/* access provided to all users, even unregistered users.               */
/*                                                                      */
/* An entry where object_id = 0 means that this user has this level of  */
/* access to ALL objects of this type.  To indicate that this user does */
/* NOT have access to ANY objects, then there shouldn't be ANY entries. */
/*                                                                      */
/* As previously mentioned, an entry where object_type = 0 means this   */
/* is a security entry for a "function" rather than an "object."        */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/* Constants - Changing it here will impact all building blocks         */
/************************************************************************/
$LS_ADMIN_THRESHOLD = 50; /* security levels at/above this value are assumed to be able to perform admin tasks */
$LS_USER_THRESHOLD = 10; /* security levels at/above this value are assumed to be able to see some data */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function set_auth                                                    */
/*                                                                      */
/* Define the level of authorization a user has to work with data for   */
/* a specific object.                                                   */
/*                                                                      */
/*----------------------------------------------------------------------*/

function set_auth($userid,$objecttype,$objectid,$authlevel)
	{ 
	/* first check to see if a record exists */
	$query = "SELECT id FROM ls_security_auth WHERE user_id='$userid' AND object_type='$objecttype' ".
		"AND object_id='$objectid'";
	$result = sql_getresult($query);
	if($row = sql_getrow($result))
		{
		/* if it exists, then update it */
		$query = "UPDATE ls_security_auth SET levelid='$authlevel' WHERE id='$row[0]'";
		sql_getresult($query);
		}
	else
		{
		/* a record doesn't exist, so create one */
		$query = "INSERT INTO ls_security_auth SET user_id='$userid', object_type='$objecttype', ".
			"object_id='$objectid', levelid='$authlevel'";
		sql_getresult($query);
		}

	} /* end of set_auth function */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function get_auth                                                    */
/*                                                                      */
/* Return the level of authorization a user has to work with data for   */
/* a specific object.                                                   */
/*                                                                      */
/* Note: This will return the higher of either the authorization for    */
/* this specific user or for any group this user is a member of or      */
/* for all users.                                                       */
/*                                                                      */
/* This returns the auth_level value between 0 and 100, not the levelid.*/
/*                                                                      */
/*----------------------------------------------------------------------*/

function get_auth($userid,$objecttype,$objectid)
	{ 
	/* assume there's no access */
	$access = 0;

	/* now get the highest value access in the database */
	$query = "SELECT auth_level FROM ls_security_auth,ls_security_levels ".
			"WHERE ls_security_auth.user_id='$userid' ".
			"AND ls_security_auth.object_type='$objecttype' ".
			"AND (ls_security_auth.object_id='$objectid' ".
			"OR ls_security_auth.object_id = '0') ".
			"AND ls_security_levels.id = ls_security_auth.levelid ".
			"ORDER BY auth_level DESC LIMIT 1";
	$result = sql_getresult($query);
	if($row = sql_getrow($result))
		{
		/* if it exists, then update it */
		$access = $row[0];
		}

	/* now check for group memberships - this can be iterative for multi-levels */
	$query = "SELECT GroupId FROM ls_group_members WHERE UserId = '$userid'";
	$result = sql_getresult($query);
	while($row = sql_getrow($result))
		{
		$thisaccess = get_auth($row[0],$objecttype,$objectid);
		if($thisaccess > $access) $access = $thisaccess;
		}

	/* now get the highest value access in the database for 'all users' */
	if($userid > 0)
		{
		$thisaccess = get_auth(0,$objecttype,$objectid);
		if($thisaccess > $access) $access = $thisaccess;
		}

	return $access;

	} /* end of get_auth function */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function get_level_array                                             */
/*                                                                      */
/* Return the auth level and level id for the highest authorization a   */
/* user has to work with data for a specific object.                    */
/*                                                                      */
/* Note: This will only return the values for the specific user.        */
/*                                                                      */
/*----------------------------------------------------------------------*/

function get_level_array($userid,$objecttype,$objectid)
	{ 
	/* assume there's no access */
	$level['access'] = 0;
	$level['id'] = 0;

	/* now get the highest value access in the database */
	$query = "SELECT auth_level,levelid FROM ls_security_auth,ls_security_levels ".
			"WHERE ls_security_auth.user_id='$userid' ".
			"AND ls_security_auth.object_type='$objecttype' ".
			"AND (ls_security_auth.object_id='$objectid' ".
			"OR ls_security_auth.object_id = '0') ".
			"AND ls_security_levels.id = ls_security_auth.levelid ".
			"ORDER BY auth_level DESC LIMIT 1";
	$result = sql_getresult($query);
	if($row = sql_getrow($result))
		{
		/* if it exists, then update it */
		$level['access'] = $row[0];
		$level['id'] = $row[1];
		}

	/* now check for groups */
	$query = "SELECT GroupId FROM ls_group_members WHERE UserId = '$userid'";
	$result = sql_getresult($query);
	while($row = sql_getrow($result))
		{
		$thislevel = get_level_array($row[0],$objecttype,$objectid);
		if($thislevel['access'] > $thislevel['access'])
			{
			$level['access'] = $thislevel['access'];
			$level['id'] = $thislevel['id'];
			}
		}

	/* now check for all users */
	if($userid > 0)
		{
		$thislevel = get_level_array(0,$objecttype,$objectid);
		if($thislevel['access'] > $level['access'])
			{
			$level['access'] = $thislevel['access'];
			$level['id'] = $thislevel['id'];
			}
		}

	return $level;

	} /* end of get_level_array function */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function get_level                                                   */
/*                                                                      */
/* Return the level id of authorization a user has to work with data    */
/* for a specific object.                                               */
/*                                                                      */
/* Note: This will return the higher of either the authorization for    */
/* this specific user or for any group this user is a member of or      */
/* for all users.                                                       */
/*                                                                      */
/*----------------------------------------------------------------------*/

function get_level($userid,$objecttype,$objectid)
	{ 
	/* get the access and level id for the user */
	$level = get_level_array($userid,$objecttype,$objectid);

	$access = $level['access'];
	$levelid = $level['id'];

	return $levelid;

	} /* end of get_level function */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function get_field_auth                                              */
/*                                                                      */
/* Indicate whether the user has read and/or write access to specific   */
/* field data for a specific object.                                    */
/*                                                                      */
/* Note: This will return a 0 for no access, a 1 for read, and          */
/*       a 2 for read+write access.                                     */
/*                                                                      */
/*----------------------------------------------------------------------*/

function get_field_auth($userid,$objecttype,$objectid,$fieldname)
	{ 
	/* assume there's no access */
	$access = 0;

	/* find out the auth_level for this object for this user */
	$auth_level = get_level($userid,$objecttype,$objectid);

	$query = "SELECT readauth,writeauth FROM ls_security_fields ".
		"WHERE level='$auth_level' AND fieldname='".addslashes($fieldname)."'";
	$result = sql_getresult($query);
	if($row = sql_getrow($result))
		{
		if($row[0] > 0) $access = 1;
		if($row[1] > 0) $access = 2;
		}

	return $access;

	} /* end of get_field_auth function */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function check_auth                                                  */
/*                                                                      */
/* Checks to make sure a user has at least a certain level of           */
/* authorization to work with data for a specific object.               */
/*                                                                      */
/*----------------------------------------------------------------------*/

function check_auth($userid,$objecttype,$objectid,$authlevel)
	{ 
	$access = get_auth($userid,$objecttype,$objectid);

	if($access >= $authlevel) return TRUE;
	else return FALSE;

	} /* end of check_auth function */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function check_function_auth                                         */
/*                                                                      */
/* Checks to make sure a user has at least a certain level of           */
/* authorization to perform a certain function.                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

function check_function_auth($userid,$objecttype,$functionname,$authlevel)
	{ 
	/* First find the entry in ls_security_functions */
	$objectid = sql_getone("SELECT id FROM ls_security_functions ".
		"WHERE object_type='$objecttype' AND lsfunction='".addslashes($functionname)."'");

	$access = get_auth($userid,0,$objectid);

	if($access >= $authlevel) return TRUE;
	else return FALSE;

	} /* end of check_function_auth function */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function check_type_auth                                             */
/*                                                                      */
/* Checks to make sure a user has at least a certain level of           */
/* authorization to work with any data for a specific type of object.   */
/*                                                                      */
/*----------------------------------------------------------------------*/

function check_type_auth($userid,$objecttype,$authlevel)
	{ 
	/* assume there's no access */
	$access = 0;

	/* now get the highest value access to objects in the database */
	$query = "SELECT auth_level,levelid FROM ls_security_auth,ls_security_levels ".
			"WHERE ls_security_auth.user_id='$userid' ".
			"AND ls_security_auth.object_type='$objecttype' ".
			"AND ls_security_levels.id = ls_security_auth.levelid ".
			"ORDER BY auth_level DESC LIMIT 1";
	$result = sql_getresult($query);
	if($row = sql_getrow($result))
		{
		/* if it exists, then update it */
		$access = $row[0];
		}

	/* now do the same for functions */
	$query = "SELECT auth_level,levelid FROM ls_security_auth,ls_security_levels,ls_security_functions ".
			"WHERE ls_security_auth.user_id='$userid' ".
			"AND ls_security_auth.object_type='0' ".
			"AND ls_security_functions.id = ls_security_auth.object_id ".
			"AND ls_security_auth.object_type='$objecttype' ".
			"AND ls_security_levels.id = ls_security_auth.levelid ".
			"ORDER BY auth_level DESC LIMIT 1";
	$result = sql_getresult($query);
	if($row = sql_getrow($result))
		{
		/* if it exists and is higher, then update it */
		if($row[0] > $access) $access = $row[0];
		}

	/* now check for group memberships - this can be iterative for multi-levels */
	$groupaccess = FALSE;
	$query = "SELECT GroupId FROM ls_group_members WHERE UserId = '$userid'";
	$result = sql_getresult($query);
	while($row = sql_getrow($result))
		{
		$thisaccess = check_type_auth($row[0],$objecttype,$authlevel);
		if($thisaccess == TRUE) 
			{
			$groupaccess = TRUE;
			break;
			}
		}

	/* now check for 'all users' */
	if($userid > 0) 
		$allaccess = check_type_auth(0,$objecttype,$authlevel);

	if(($access >= $authlevel) OR $groupaccess OR $allaccess) return TRUE;
	else return FALSE;

	} /* end of check_type_auth function */

/*----------------------------------------------------------------------*/
/*                                                                      */
/* Function create_admin_menu                                           */
/*                                                                      */
/* Outputs the lines of HTML/CSS to display the menu of administrative  */
/* options given the user's level of access for different object types  */
/*                                                                      */
/* Note: $cur_objecttype indicates the type of object currently being   */
/*    managed by the user - ie. that menu object shouldn't be selectable*/
/*    This would be in the form of the name of admin program            */
/*                                                                      */
/* This makes some important assumptions about programs to manage       */
/* Living Stones objects - namely that all programs to manage objects   */
/* will be named ls_manage_xxxx where xxxx is the "object_desc" in all  */
/* lower case with spaces converted to underscores, while the menu      */
/* itself will be labeled with "Xxxx" where the first letter in each    */
/* word will be capitalized.                                            */
/*                                                                      */
/*----------------------------------------------------------------------*/

function create_admin_menu($userid,$cur_objecttype,$admin_threshold)
	{ 
	echo "<div id=\"navigation\">\n";
	echo "<ul id=\"navlist\">\n";
	/* loop through all possible object types (alphabetical) */
	$query = "SELECT id,object_desc FROM ls_security_objects WHERE id>0 ORDER BY object_desc ASC";
	$result = sql_getresult($query);
	while($row = sql_getrow($result))
		{
		if(check_type_auth($userid,$row[0],$admin_threshold))
			{
			$object_desc = $row[1];

			/* the program name will be ls_manage_xxxx where xxxx is the object_desc (lower case) */
			$prog_name = "ls_manage_".strtolower($object_desc).".php";
			$prog_name = str_replace(" ","_",$prog_name);

			/* the menu label will be Xxxx where Xxxx is the object_desc */
			/* with the first letter of each word capitalized            */
			$parts = explode(" ",$object_desc);
			for($i = 0;$i < count($parts);$i++)
				{
				$first = strtoupper(substr($parts[$i],0,1));
				$last = substr($parts[$i],1);
				$parts[$i] = $first.$last;
				} /* looping through words in object desc */
			$menulabel = implode(" ",$parts);

			if($prog_name == $cur_objecttype)
				{
				echo "  <li id=\"current\">$menulabel</li>\n";
				$thisscript = $prog_name;
				}
			else
				echo "  <li><a href=\"$prog_name\">$menulabel</a></li>\n";
			} /* end if check_type_auth - this user can manage this object type */
		} /* end while looping through object types */

	/* now add either a login or logout option */
	if(strlen($userid) < 1) 
		echo "  <li><a href=\"ls_login.php?return_path=$thisscript\">Login</a></li>\n";
	else
		echo "  <li><a href=\"ls_login.php?logout=1\">Logout</a></li>\n";

	echo "</ul>\n";
	echo "</div> <!-- navigation -->\n";
	} /* end of create_admin_menu function */
?>
